package com.safeway.epe.service;

import com.safeway.epe.client.ProducerServiceProxy;
import com.safeway.epe.domain.ConsumerPayload;
import com.safeway.epe.domain.Offsets;
import com.safeway.epe.domain.TransactionPayload;
import com.safeway.epe.domain.TransactionRecorder;
import com.safeway.epe.repository.TransactionRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import java.util.ArrayList;
import java.util.List;

@Component
public class ProducerService
{
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private TransactionService transactionService;

    @Autowired
    private ProducerServiceProxy producerServiceProxy;

    @Autowired
    private TransactionRepository transactionRepository;

    public List<Offsets> produce(TransactionPayload request, String topic, TransactionRecorder transactionRecorder, boolean isReprocesed)
    {
        logger.info("TransactionId :" + transactionRecorder.getUuid());
        ResponseEntity<ConsumerPayload> response = null;
        try
        {
            response = producerServiceProxy.produceTransactionService(topic, request);
        }
        catch(Exception e)
        {
            logger.error("Exception " + e.getMessage());
        }
        logger.info("response ::" + response);

        if(response != null && (response.getStatusCodeValue() == 200 || response.getStatusCode().is2xxSuccessful()))
        {
            logger.info("Data is inserted to topic");
            return response.getBody().getOffsetsList();
        }
        else
        {
            transactionRepository.updateProcessedFlagByTransactionId(false,transactionRecorder.getUuid());
            errorResponse(transactionRecorder, "failed to insert data to topic", false);
            logger.info("failed to insert data to topic");
            return new ArrayList<>();
        }
    }

    private ResponseEntity<String> errorResponse(TransactionRecorder transactionRecorder, String exceptionMessage, boolean isReprocesed) {
        if(!isReprocesed){
            logger.error("Error response : "+ exceptionMessage);
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body("Error");
    }

}
