package com.safeway.epe.service;

import com.safeway.epe.client.ProducerServiceProxy;
import com.safeway.epe.domain.ConsumerPayload;
import com.safeway.epe.domain.Offsets;
import com.safeway.epe.domain.TransactionPayload;
import com.safeway.epe.domain.TransactionRecorder;
import org.junit.jupiter.api.*;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;

import java.util.Arrays;
import java.util.List;

@DisplayName("Producer_Service_Test_Class")
public class ProducerServiceTest {

    Logger logger = LoggerFactory.getLogger(this.getClass());

    @InjectMocks
    ProducerService producerService;
    @Mock
    ProducerServiceProxy mockProducerServiceProxy;

    private final String topic = "trasactionRecordTopic";
    private final boolean isReprocesed = true;
    List<Offsets> mockOffsetsList;
    TransactionPayload request;
    TransactionRecorder transactionRecorder;

    @BeforeEach
    public void setUp() throws Exception {
        logger.info("After Each Method Executed");
        MockitoAnnotations.initMocks(this);
        request = new TransactionPayload();
        transactionRecorder = new TransactionRecorder();
    }

    @Test
    public void testProduce(){
        Mockito.when(mockProducerServiceProxy.produceTransactionService(Mockito.anyString(),Mockito.any())).thenReturn(mockConsumerPayloadResponse());
        List<Offsets> offsetsList = producerService. produce(request, topic,transactionRecorder , isReprocesed);
        Assertions.assertEquals(offsetsList,mockOffsetsList);
    }
    private ResponseEntity<ConsumerPayload> mockConsumerPayloadResponse(){
        ConsumerPayload entity = new ConsumerPayload();
        Offsets offsets = new Offsets();
        offsets.setOffset("1");
        offsets.setPartition("0");
        mockOffsetsList = Arrays.asList(offsets);
        entity.setOffsetsList(mockOffsetsList);
        return ResponseEntity.ok(entity);
    }

    @AfterEach
    public void tearDown(){
        logger.info("After Each Method Executed");
    }
}
