package com.safeway.epe.service;

import com.safeway.epe.config.ConfigProps;
import com.safeway.epe.domain.Offsets;
import com.safeway.epe.domain.TransactionRecorder;
import com.safeway.epe.repository.TransactionRepository;
import com.safeway.epe.util.TransactionServiceArgumentProvider;
import org.junit.jupiter.api.*;
import org.junit.jupiter.api.extension.ExtendWith;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ArgumentsSource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;

import java.util.*;

@DisplayName("Transaction_Service_Impl_Test_Class")
public class TransactionServiceImplTest {

    Logger logger = LoggerFactory.getLogger(this.getClass());

    @InjectMocks
    TransactionServiceImpl transactionServiceImpl;
    @Mock
    private TransactionRepository mockTransactionRepository;
    @Mock
    private ProducerService mockProducerService;
    @Mock
    private ConfigProps configProps;

    @BeforeEach
    public void setUp() throws Exception {
        logger.info("Before Each Method Executed");
        MockitoAnnotations.initMocks(this);
    }

    @ParameterizedTest
    @ArgumentsSource(TransactionServiceArgumentProvider.class)
    @DisplayName("Produce_Message_to_Kafka_Topic_Success_Scenario")
    public void testProduceMessage(TransactionRecorder transactionRecorder){

        String transactionId = "997a9ed1-fb45-46d6-baee-0230f1da93dd";
        UUID uuid = UUID.fromString(transactionId);

        Optional<TransactionRecorder> testTransactionRecord = Optional.of(transactionRecorder);
        Mockito.when(mockTransactionRepository.findById(uuid)).thenReturn(testTransactionRecord);
        ResponseEntity<List<Offsets>> offsetsList = transactionServiceImpl.produceMessage(transactionId);
        Assertions.assertEquals(offsetsList.getStatusCode(), HttpStatus.OK);

    }

    @Test
    @DisplayName("Produce_Message_to_Kafka_Topic_Exception_Scenario")
    public void testProduceMessage_Exception(){

        String transactionId = "997a9ed1-fb45-46d6-baee-0230f1da93dd";
        UUID uuid = UUID.fromString(transactionId);

        Optional<TransactionRecorder> testTransactionRecord = Optional.empty();
        Mockito.when(mockTransactionRepository.findById(uuid)).thenReturn(testTransactionRecord);
        ResponseEntity<List<Offsets>> responseOffsetsList = transactionServiceImpl.produceMessage(transactionId);
        Assertions.assertEquals(responseOffsetsList.getStatusCode(), HttpStatus.NOT_FOUND);
        Assertions.assertEquals(responseOffsetsList.getBody(), Arrays.asList(new Offsets()));

    }

    @AfterEach
    public void tearDown(){
        logger.info("After Each Method Executed");
    }

}

